@@ -20,7 +20,7 @@ from typing import (
 from types import TracebackType
 from pathlib import Path
 from enum import Enum
-from subprocess import Popen, run, PIPE
+from subprocess import Popen, run, PIPE, CalledProcessError
 from collections import defaultdict
 from tempfile import TemporaryDirectory
 
@@ -32,6 +32,9 @@ class MissingObject(Exception):
         Exception.__init__(self, f"Object {ref} does not exist")
 
 
+T = TypeVar("T")  # pylint: disable=invalid-name
+
+
 class Oid(bytes):
     """Git object identifier"""
 
@@ -60,9 +63,7 @@ class Oid(bytes):
     def for_object(cls, tag: str, body: bytes):
         """Hash an object with the given type tag and body to determine its Oid"""
         hasher = hashlib.sha1()
-        hasher.update(
-            tag.encode("ascii") + b" " + str(len(body)).encode("ascii") + b"\0" + body
-        )
+        hasher.update(tag.encode() + b" " + str(len(body)).encode() + b"\0" + body)
         return cls(hasher.digest())
 
     def __repr__(self) -> str:
@@ -205,6 +206,24 @@ class Repository:
             return prog.stdout[:-1]
         return prog.stdout
 
+    def config(self, setting: str, default: T) -> Union[bytes, T]:
+        try:
+            return self.git("config", "--get", setting)
+        except CalledProcessError:
+            return default
+
+    def bool_config(self, config: str, default: T) -> Union[bool, T]:
+        try:
+            return self.git("config", "--get", "--bool", config) == b"true"
+        except CalledProcessError:
+            return default
+
+    def int_config(self, config: str, default: T) -> Union[int, T]:
+        try:
+            return int(self.git("config", "--get", "--int", config))
+        except CalledProcessError:
+            return default
+
     def __enter__(self) -> "Repository":
         return self
 
@@ -217,6 +236,9 @@ class Repository:
         if self._tempdir:
             self._tempdir.__exit__(exc_type, exc_val, exc_tb)
 
+        self._catfile.terminate()
+        self._catfile.wait()
+
     def get_tempdir(self) -> Path:
         """Return a temporary directory to use for modifications to this repository"""
         if self._tempdir is None:
@@ -243,9 +265,9 @@ class Repository:
         if committer is None:
             committer = self.default_committer
 
-        body = b"tree " + tree.oid.hex().encode("ascii") + b"\n"
+        body = b"tree " + tree.oid.hex().encode() + b"\n"
         for parent in parents:
-            body += b"parent " + parent.oid.hex().encode("ascii") + b"\n"
+            body += b"parent " + parent.oid.hex().encode() + b"\n"
         body += b"author " + author + b"\n"
         body += b"committer " + committer + b"\n"
         body += b"\n"
@@ -280,11 +302,11 @@ class Repository:
             ref = ref.hex()
 
         # Write out an object descriptor.
-        self._catfile.stdin.write(ref.encode("ascii") + b"\n")
+        self._catfile.stdin.write(ref.encode() + b"\n")
         self._catfile.stdin.flush()
 
         # Read in the response.
-        resp = self._catfile.stdout.readline().decode("ascii")
+        resp = self._catfile.stdout.readline().decode()
         if resp.endswith("missing\n"):
             # If we have an abbreviated hash, check for in-memory commits.
             try:
@@ -412,7 +434,7 @@ class GitObj:
             stdin=self.body,
         )
 
-        assert Oid.fromhex(new_oid.decode("ascii")) == self.oid
+        assert Oid.fromhex(new_oid.decode()) == self.oid
         self.persisted = True
         return self.oid
 
@@ -719,8 +741,11 @@ class Index:
 class Reference(Generic[GitObjT]):  # pylint: disable=unsubscriptable-object
     """A git reference"""
 
+    shortname: str
+    """Short unresolved reference name, e.g. 'HEAD' or 'master'"""
+
     name: str
-    """Git reference name, e.g. 'HEAD' or 'refs/heads/master'"""
+    """Resolved reference name, e.g. 'refs/tags/1.0.0' or 'refs/heads/master'"""
 
     target: Optional[GitObjT]
     """Referenced git object"""
@@ -735,7 +760,7 @@ class Reference(Generic[GitObjT]):  # pylint: disable=unsubscriptable-object
 
     def __init__(self, obj_type: Type[GitObjT], repo: Repository, name: str):
         self._type = obj_type
-        self.name = name
+        self.name = repo.git("rev-parse", "--symbolic-full-name", name).decode()
         self.repo = repo
         self.refresh()
 
